#IF(
    #DEFINED(#Shop.GoogleSmartShoppingConfig)
    AND #DEFINED(#Shop.GoogleSmartShoppingConfig.AdsCampaignConversionId)
    AND #DEFINED(#Shop.GoogleSmartShoppingConfig.AdsCampaignConversionLabel)
    AND #DEFINED(#Shop.GoogleSmartShoppingConfig.AdsCampaignRemarketingId)
)
  (function () {

    const adsCampaignConversionId = "#Shop.GoogleSmartShoppingConfig.AdsCampaignConversionId";
    const adsCampaignConversionLabel = "#Shop.GoogleSmartShoppingConfig.AdsCampaignConversionLabel";
    const adsCampaignRemarketingId = "#Shop.GoogleSmartShoppingConfig.AdsCampaignRemarketingId";

    var googlePortalsLocale = [];
    #LOOP(#Shop.PortalConfigs)
      #IF(#Portal.Alias EQ "GoogleBase")
        googlePortalsLocale.push("#PortalSite.LocaleID")
      #ENDIF
    #ENDLOOP

    var availableCountries = {}
      #LOOP(#Shop.ShopCountries)
        availableCountries["#Code2"] = "#English"
      #ENDLOOP

    function gtag() {
      dataLayer.push(arguments);
    }

    if (! (window.dataLayer && document.getElementById('GAW_googletagmanager'))) {
        let googleSmartShoppingScriptElement = document.createElement('script')
        googleSmartShoppingScriptElement.src = 'https://www.googletagmanager.com/gtag/js?id=' + adsCampaignConversionId
        googleSmartShoppingScriptElement.id = 'GSG_googletagmanager'
        document.body.appendChild(googleSmartShoppingScriptElement)

        window.dataLayer = window.dataLayer || [];
        gtag('js', new Date());
    }

    gtag('config', adsCampaignConversionId);
    gtag('event', 'page_view',{
      send_to: adsCampaignConversionId
    })


    return function () {
      if (window.eComEventTarget) {
        window.eComEventTarget.addEventListener('product', onProductViewEvent)
        window.eComEventTarget.addEventListener('cart:add', onCartAddEvent)
        window.eComEventTarget.addEventListener('cart', onCartViewEvent)
        window.eComEventTarget.addEventListener('order:completed', onOrderCompletedEvent)
      }
    }();

  function onProductViewEvent (event) {
    var product = event.detail.product.toJS()

    if (!product || !product.price || product.price.amount <= 0) {
      return
    }

    let googleProduct = getGoogleProduct(product)

    const eventParameters = {
      'send_to': adsCampaignConversionId,
      'ecomm_pagetype': 'product',
      'value': googleProduct.price,
      'items': [googleProduct]
    }

    gtag('event', 'view_item', eventParameters)
  }

  function onCartAddEvent (event) {
    var product = event.detail.product
    var eventQuantity = event.detail.quantity

    if (!product || !product.price || product.price.amount <= 0 || eventQuantity <= 0) {
      return
    }

    let googleProduct = getGoogleProduct(product)
    googleProduct.quantity = eventQuantity

    const eventParameters = {
      'send_to': adsCampaignConversionId,
      'ecomm_pagetype': 'cart',
      'value': (googleProduct.price * eventQuantity).toFixed(2),
      'items': [googleProduct]
    }

    gtag('event', 'add_to_cart', eventParameters)
  }

  function onCartViewEvent (event) {
    var cart = event.detail.cart
    if (!cart || !cart.productLineItems || cart.grandTotal.amount <= 0) {
      return
    }

    let googleProducts = cart.productLineItems.map(product => {
      const modifiedProduct = Object.assign({}, product, {price: product.listPrice});

      let googleProduct = getGoogleProduct(modifiedProduct)
      googleProduct.quantity = product.quantity
      return googleProduct
    })

    const eventParameters = {
      'send_to': adsCampaignConversionId,
      'ecomm_pagetype': 'cart',
      'value': cart.grandTotal.amount,
      'items': googleProducts
    }

    gtag('event', 'page_view', eventParameters)
  }

  function onOrderCompletedEvent (event) {
    var order = event.detail
    if (!order || !order.lineItemContainer || order.lineItemContainer.grandTotal.amount <= 0) {
      return
    }

    let googleProducts = order.lineItemContainer.productLineItems.map(product => {
      const modifiedProduct = Object.assign({}, product, {price: product.listPrice});

      let googleProduct = getGoogleProduct(modifiedProduct)
      googleProduct.quantity = product.quantity.amount
      return googleProduct
    })

    const eventParameters = {
      'ecomm_pagetype': 'purchase',
      'send_to': adsCampaignConversionLabel,
      'transaction_id': order.orderNumber,
      'value': order.lineItemContainer.grandTotal.amount,
      'currency': order.lineItemContainer.grandTotal.currency,
      'country': getShopperCountryCodeFromOrder(order),
      'items': googleProducts
    }

    gtag('event', 'purchase', eventParameters)
  }

  function getGoogleProduct(product) {
    return {
      'id': getGoogleProductId(product.sku),
      'price': product.price ? product.price.amount : product.listPrice.amount,
      'google_business_vertical': 'retail',
      'name': product.name
    }
  }

  function getGoogleProductId(productCode) {
    if (googlePortalsLocale.length === 0) {
        return productCode;
    }

    if (googlePortalsLocale.length === 1) {
      return (productCode + '_' + googlePortalsLocale[0])
    }

    const shopperLocale = getShopperLocale()
    const matchingPortalsLocale = googlePortalsLocale.filter(portalLocale => portalLocale.includes(shopperLocale))

    if (matchingPortalsLocale.length >= 1) {
      return (productCode + '_' + matchingPortalsLocale[0])
    }

    return (productCode + '_' + googlePortalsLocale[0])
  }

  function getShopperLocale() {
    let countryCode = ''
    if (document.documentElement && document.documentElement.lang) {
      countryCode = document.documentElement.lang
    }
    return countryCode
  }

  function getShopperCountryCodeFromOrder(order) {
    let shopperCountryCode = "#Shop.Address.Country.Code2"
    if (order.shippingAddress && order.shippingAddress.country) {
      return getCountryCodeFromCountryName(order.shippingAddress.country)
    }
    if (order.billingAddress && order.billingAddress.country) {
      return getCountryCodeFromCountryName(order.billingAddress.country)
    }

    return shopperCountryCode
  }

  function getCountryCodeFromCountryName(orderCountryName) {
    let shopperCountryCode = "#Shop.Address.Country.Code2"
    for (let availableCountryCode in availableCountries) {
      if (orderCountryName.includes(availableCountries[availableCountryCode])) {
        return availableCountryCode
      }
    }
    return shopperCountryCode
  }

  }())
#ENDIF
