#==============================================================================
# package      DE_EPAGES::GoogleSmartShopping::UI::ProductFeed
# state        private
#==============================================================================
package DE_EPAGES::GoogleSmartShopping::UI::ProductFeed;
use base DE_EPAGES::Presentation::UI::Object;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );

use DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping );

use strict;

#==============================================================================
# Event handlers
#==============================================================================
sub UnityViewGoogleSmartShoppingProductFeed {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    unless ($self->_areProductUploadRequirementsMet($Shop)) {
        $Servlet->vars('IsForbiddenProductUpload', 1);
        return $self->SUPER::View($Servlet);
    }

    my $hProductsStatistics = $self->_getProductStatistics($Shop);
    $Servlet->vars('ProductStatuses', $hProductsStatistics);

    return $self->SUPER::View($Servlet);
}


#----------------------------------------------------------------------------------------
sub _areProductUploadRequirementsMet {
    my $self = shift;
    my ($Shop) = @_;

    return unless (defined($Shop->get('Address')->get('Country')));
    return unless ($Shop->existsChild('GoogleSmartShoppingConfig'));
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');

    return unless ($GoogleSmartShoppingConfig->areGoogleMerchantCenterRequirementsMet());
    # most important is that we have Merchant Center account connected
    return unless (isValid($GoogleSmartShoppingConfig->get('MerchantCenterAccountID')));

    return 1;
}

sub _getProductStatistics {
    my $self = shift;
    my ($Shop) = @_;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
    my $hResult;
    eval {
        $hResult = $Service->getRatedProductsStatistics();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping("Google ProductLevelIssues request failed: ", {
            Error => $Error,
            ErrorMessage => $Error->message(),
        });
    }
    return $hResult;
}

1;
