#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::ProductFeed
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the GoogleSmartShopping ProductFeed tab
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::ProductFeed;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( String HashRef );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled );

use Test::More;

use strict;

use constant MERCHANT_CENTER_ID => 503220055;

sub test {
    my $self = shift;

    $self->test_UnityViewGoogleSmartShoppingProductFeed_success();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_NoShopAddress();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_NoSmarshoppingConfig();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_MerchantCenterRequirementsNotMet();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_NoMerchantCenterAccountID();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_Success_ProductsStatistics();

    done_testing();
}

sub test_UnityViewGoogleSmartShoppingProductFeed_success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $IsGoogleSmartShoppingConfigAvailable = 1;

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);
    my $MockedShop = $self->_createShop($IsGoogleSmartShoppingConfigAvailable, $MockedAddress, $MockedGoogleConfig);

    my $MockedServlet = $self->_createServlet($MockedShop);

    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsForbiddenProductUpload'), undef, $SubTestName . ' - Verify that Servlet error IsForbiddenProductUpload is NOT defined.' );

}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_NoShopAddress {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);
    my $IsGoogleSmartShoppingConfigAvailable = 1;

    my $hCountry = undef;
    my $MockedAddress = $self->_createAddress($hCountry);
    my $MockedShop = $self->_createShop($IsGoogleSmartShoppingConfigAvailable, $MockedAddress,  $MockedGoogleConfig);

    my $MockedServlet = $self->_createServlet($MockedShop);

    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsForbiddenProductUpload'), 1, $SubTestName . ' - Verify that Servlet error IsForbiddenProductUpload is TRUE.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_NoSmarshoppingConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => undef,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $IsGoogleSmartShoppingConfigAvailable = 1;

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);
    my $MockedShop = $self->_createShop($IsGoogleSmartShoppingConfigAvailable, $MockedAddress, $MockedGoogleConfig);

    my $MockedServlet = $self->_createServlet($MockedShop);

    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsForbiddenProductUpload'), 1, $SubTestName . ' - Verify that Servlet error IsForbiddenProductUpload is TRUE.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_MerchantCenterRequirementsNotMet {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 0,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);
    my $IsGoogleSmartShoppingConfigAvailable = 1;
    my $MockedShop = $self->_createShop($IsGoogleSmartShoppingConfigAvailable, $MockedAddress, $MockedGoogleConfig);

    my $MockedServlet = $self->_createServlet($MockedShop);

    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsForbiddenProductUpload'), 1, $SubTestName . ' - Verify that Servlet error IsForbiddenProductUpload is TRUE.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_NoMerchantCenterAccountID {

}

sub test_UnityViewGoogleSmartShoppingProductFeed_Success_ProductsStatistics {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $IsGoogleSmartShoppingConfigAvailable = 1;

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);
    my $MockedShop = $self->_createShop($IsGoogleSmartShoppingConfigAvailable, $MockedAddress, $MockedGoogleConfig);

    my $Servlet = $self->_createServlet($MockedShop);

    my $ExpectedProductsStatistics = {
        TooManyDisapproved  => 0,
        TooManyPending      => 0,
        NotEnoughActive     => 0,
        total               => 5,
        active              => 5,
        pending             => 0,
        disapproved         => 0
    };

    my $GoogleApiService = $self->_createGoogleApiService({
        RatedProductsStatistics => $ExpectedProductsStatistics
    });

    my $hParams = {
        'GoogleApiService' => $GoogleApiService,
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->UnityViewGoogleSmartShoppingProductFeed($Servlet);

    ok(WasCalled($GoogleApiService, 'getRatedProductsStatistics'),
        $SubTestName . ' - Verify that service method _getProductStatistics was called.');
    is_deeply(
        $Servlet->vars('ProductStatuses'),
        $ExpectedProductsStatistics,
        "$SubTestName - Verify that ProductStatistics was correctly set in servlet."
    );
}
#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($hParams ) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::ProductFeed');

    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    if (defined $hParams->{'GoogleApiService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::new')
            ->when(String(), HashRef())
            ->thenReturn($hParams->{'GoogleApiService'});
    }

    $MockifySut->spy('_areProductUploadRequirementsMet')->whenAny();

    return $MockifySut->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($IsGoogleSmartShoppingConfigAvailable, $MockedAddress, $MockedGoogleConfig) = @_;

    my $hAttributes = {
        'Address' => $MockedAddress,
        'GoogleSmartShoppingConfig' => $MockedGoogleConfig,
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    $MockifyShop->addMockWithReturnValueAndParameterCheck('existsChild', $IsGoogleSmartShoppingConfigAvailable, [{'string'=>'GoogleSmartShoppingConfig'}]);

    return $MockifyShop->getMockObject();
}

sub _createGoogleConfig {
    my $self = shift;
    my ($hConfig) = @_;

    $hConfig->{'AreGoogleMerchantCenterRequirementsMet'} //= 0;

    my $hAttributes = {
        ChecklistBillingTOC                => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistContactInfo               => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistGoogleAdsPolicies         => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistCookieNotice              => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistReturnPolicy              => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        MerchantCenterAccountID            => $hConfig->{'MerchantCenterAccountID'} // undef,
        GoogleAdsAccountID                 => $hConfig->{'GoogleAdsAccountID'} // undef,
        AdsCampaignId                      => $hConfig->{'AdsCampaignId'} // undef,
        MerchantCenterMultiClientAccountID => $hConfig->{'MerchantCenterMultiClientAccountID'} // undef,
        AreMerchantCenterAndAdsLinked      => $hConfig->{'AreMerchantCenterAndAdsLinked'} // 1,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($Shop) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);

    return $MockifyServlet->getMockObject();
}

sub _createAddress {
    my $self = shift;
    my ($hCountry) = @_;

    my $hParameterList = {
        'Country'   => $hCountry ,
    };

    my $MockAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Address::API::Object::Address',
        $hParameterList
    );

    return $MockAddress->getMockObject();
}

sub _createGoogleApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService');

    $Mockify->mock('getRatedProductsStatistics')
        ->when()
        ->thenReturn($hServiceResponse->{'RatedProductsStatistics'} // {}
    );
    return $Mockify->getMockObject();

}

__PACKAGE__->runScript();
