package DE_EPAGES::Zettle::t::03_ZettleVariationOptionHelper;

use base DE_EPAGES::Shop::API::TestImportXML;
use Test::More;
use strict;
use warnings;

use DE_EPAGES::Product::API::Shop qw(GetProduct);

use DE_EPAGES::Zettle::API::ImageHelper qw(GetImageUrls);

sub shopAlias { return 'TestShop'; }
sub importXMLPath { return 'TestShop.xml'; }

sub test {
    my $self = shift;
    my $Shop = $self->shop;

    # and test with the product also
    my $UseMaster = 1;
    my $ProductWithVariations = GetProduct($Shop, 'be_40401', $UseMaster);
    ok($ProductWithVariations, 'Product with variations loaded');

    # _BuildImageUrl tests
    my $urls = [
       {'in' => undef                           , 'excpected' => undef},
       {'in' => ''                              , 'excpected' => undef},
       {'in' => 'https://example.com/asdf.jpg'  , 'excpected' => 'https://example.com/asdf.jpg'},
       {'in' => 'https://example.com/ä ö.jpg'   , 'excpected' => 'https://example.com/%C3%A4%20%C3%B6.jpg'},
       {'in' => 'asdf.jpg'                      , 'excpected' => $Shop->get('ProtocolAndServerSSL') . $ProductWithVariations->get('WebPath') . '/asdf.jpg' },
    ];

    foreach my $url (@$urls) {
       ok(DE_EPAGES::Zettle::API::ImageHelper::_BuildImageUrl($ProductWithVariations, $url->{'in'}) eq $url->{'excpected'}, $url->{'in'}); ## no critic qw(ProtectPrivateSubs)
    }


    # and with the image attributes
    # test product has two different image specifications
    my $aImageUrls = GetImageUrls($ProductWithVariations, 'ImageLarge');
    ok(defined($aImageUrls), 'got image from shop');
    ok(scalar(@$aImageUrls) == 1, 'one image');

    # image medium has a url image
    my $aThirdPartyImageUrls = GetImageUrls($ProductWithVariations, 'ImageMedium');
    ok(defined($aThirdPartyImageUrls), 'got image from 3rd party');

    my $FirstThirdPartyImageUrl = pop(@$aThirdPartyImageUrls);
    my @matches = $FirstThirdPartyImageUrl =~ /(\w:\/\/)/g;
    ok(scalar(@matches) == 1, 'proto:// only once in third party url');

    # image small has no image
    my $aNoImage = GetImageUrls($ProductWithVariations, 'ImageSmall');
    ok(defined($aNoImage), 'still get an array');
    ok(scalar(@$aNoImage) == 0, 'but it is empty');

    done_testing();
}

__PACKAGE__->runScript;
1;
